class list_utils:
    def __init__(self):
        pass

    def reverse(self, input_list):
        return input_list[::-1]   
    
    def sort(self, input_list):
        return sorted(input_list)
    
    def reverse(self, input_list):
        return input_list[::-1]   
   
    def unique(self, input_list):
        return list(set(input_list))
    
    
    
if __name__ == "__main__":
    list_utils()
    

