"""
Elan - Pratik Python yardımcı kütüphanesi
"""

from elan.main import elan
from elan.math_utils import math_utils
from elan.string_utils import string_utils
from elan.image_utils import image_utils
from elan.list_utils import list_utils
from elan.video_utils import video_utils

__version__ = "0.1.0"
__author__ = "Efekan Nefesoğlu"
__license__ = "MIT"